@@ -20,62 +20,135 @@ An instance of a ``LabelMapper`` class is passed to `RegionsSelector`.
 The labels are used by `RegionsSelector` to match inputs to transforms.
 Finally, `RegionsSelector` evaluates the transforms on the corresponding inputs.
 Label mappers and transforms take the same inputs as
-`RegionsSelector`. The inputs should be filtered appropriately using the ``inputs_mapping``
-argument which is ian instance of `~astropy.modeling.mappings.Mapping`.
-The transforms in "transform_selector" should have the same number of inputs and outputs.
+`RegionsSelector`. The inputs should be filtered appropriately using the
+``inputs_mapping`` argument which is ian instance of
+`~astropy.modeling.mappings.Mapping`.  The transforms in "transform_selector"
+should have the same number of inputs and outputs.
 
-This is illustrated below using two regions, labeled 1 and 2 ::
+This is illustrated below using two regions, labeled 1 and 2
 
-    +-----------+
-    | +-+       |
-    | | |  +-+  |
-    | |1|  |2|  |
-    | | |  +-+  |
-    | +-+       |
-    +-----------+
 
-::
+.. tabs::
+    .. tab:: Diagram
 
-                              +--------------+
-                              | label mapper |
-                              +--------------+
-                                ^       |
-                                |       V
-                      ----------|   +-------+
-                      |             | label |
-                    +--------+      +-------+
-           --->     | inputs |          |
-                    +--------+          V
-                         |          +--------------------+
-                         |          | transform_selector |
-                         |          +--------------------+
-                         V                  |
-                    +-----------+           |
-                    | transform |<-----------
-                    +------------+
-                         |
-                         V
-                    +---------+
-                    | outputs |
-                    +---------+
+        .. graphviz::
+            :align: center
+
+            digraph regions {
+                node [shape=box, style=filled, fillcolor=lightyellow, fontname="Helvetica", margin="0.3,0.3"];
+
+                subgraph cluster_detector {
+                    label="Detector";
+                    style=filled;
+                    color=lightgray;
+                    fontname="Helvetica";
+
+                    region1 [label="1", shape=box, height=2, width=1];
+                    region2 [label="2", shape=box, height=1, width=1];
+
+                    {rank=same; region1 region2}
+                }
+            }
+
+    .. tab:: ASCII
+
+        .. code-block::
+
+            +-----------+
+            | +-+       |
+            | | |  +-+  |
+            | |1|  |2|  |
+            | | |  +-+  |
+            | +-+       |
+            +-----------+
+
+
+.. tabs::
+    .. tab:: Diagram
+
+        .. graphviz::
+            :align: center
+
+            digraph selector_workflow {
+                splines="ortho"
+                node [shape=box, style=filled, fillcolor=lightblue, fontname="Arial", margin="0.3,0.3"];
+
+                // Nodes
+                in [shape=point, width=0, height=0];
+                inputs [label="inputs", fillcolor=lightgreen];
+                label_mapper [label="label mapper"];
+                label [label="label"];
+                transform_selector [label="transform_selector"];
+                transform [label="transform"];
+                outputs [label="outputs", fillcolor=lightgreen];
+
+                // Invisible nodes for alignment
+                invisible1 [style=invis, shape=point];
+                invisible2 [style=invis, shape=point];
+
+                // Main flow
+                in ->inputs -> transform -> outputs;
+                inputs -> invisible1 [style=invis];
+                inputs -> label_mapper [constraint=false];
+                label_mapper -> label;
+                label -> transform_selector;
+                transform_selector -> transform;
+                invisible2 -> transform [style=invis];
+
+                // Layout adjustments
+                {rank=same; label_mapper; invisible1;}
+                {rank=same; transform_selector; invisible2;}
+            }
+
+    .. tab:: ASCII
+
+        .. code-block::
+
+                               +--------------+
+                               | label mapper |
+                               +--------------+
+                                 ^       |
+                                 |       V
+                       ----------|   +-------+
+                       |             | label |
+                     +--------+      +-------+
+            --->     | inputs |          |
+                     +--------+          V
+                          |          +--------------------+
+                          |          | transform_selector |
+                          |          +--------------------+
+                          V                  |
+                     +-----------+           |
+                     | transform |<-----------
+                     +------------+
+                          |
+                          V
+                     +---------+
+                     | outputs |
+                     +---------+
 
 
 The base class _LabelMapper can be subclassed to create other
 label mappers.
+"""  # noqa: E501
 
-
-"""
+import contextlib
 import warnings
+
 import numpy as np
-from astropy.modeling.core import Model
 from astropy.modeling import models as astmodels
+from astropy.modeling.core import Model
 
 from . import region
 from .utils import RegionError, _toindex
 
-
-__all__ = ['LabelMapperArray', 'LabelMapperDict', 'LabelMapperRange', 'RegionsSelector',
-           'LabelMapper']
+__all__ = [
+    "LabelMapper",
+    "LabelMapperArray",
+    "LabelMapperDict",
+    "LabelMapperRange",
+    "RegionsSelector",
+]
 
 
 def get_unique_regions(regions):
@@ -85,25 +158,24 @@ def get_unique_regions(regions):
 
         try:
             unique_regions.remove(0)
-            unique_regions.remove('')
-        except ValueError:
-            pass
-        try:
             unique_regions.remove("")
         except ValueError:
             pass
+        with contextlib.suppress(ValueError):
+            unique_regions.remove("")
     elif isinstance(regions, dict):
         unique_regions = []
-        for key in regions.keys():
+        for key in regions:
             unique_regions.append(regions[key](key))
     else:
-        raise TypeError("Unable to get unique regions.")
+        msg = "Unable to get unique regions."
+        raise TypeError(msg)
     return unique_regions
 
 
 class LabelMapperArrayIndexingError(Exception):
     def __init__(self, message):
-        super(LabelMapperArrayIndexingError, self).__init__(message)
+        super().__init__(message)
 
 
 class _LabelMapper(Model):
@@ -127,11 +199,12 @@ class _LabelMapper(Model):
     name : str
         The name of this transform.
     """
+
     def __init__(self, mapper, no_label, inputs_mapping=None, name=None, **kwargs):
         self._no_label = no_label
         self._inputs_mapping = inputs_mapping
         self._mapper = mapper
-        super(_LabelMapper, self).__init__(name=name, **kwargs)
+        super().__init__(name=name, **kwargs)
 
     @property
     def mapper(self):
@@ -146,11 +219,11 @@ class _LabelMapper(Model):
         return self._no_label
 
     def evaluate(self, *args):
-        raise NotImplementedError("Subclasses should implement this method.")
+        msg = "Subclasses should implement this method."
+        raise NotImplementedError(msg)
 
 
 class LabelMapperArray(_LabelMapper):
-
     """
     Maps array locations to labels.
 
@@ -181,21 +254,21 @@ class LabelMapperArray(_LabelMapper):
     fittable = False
 
     def __init__(self, mapper, inputs_mapping=None, name=None, **kwargs):
-        if mapper.dtype.type is not np.unicode_:
+        if mapper.dtype.type is not np.str_:
             mapper = np.asanyarray(mapper, dtype=int)
             _no_label = 0
         else:
             _no_label = ""
-        super(LabelMapperArray, self).__init__(mapper, _no_label, name=name, **kwargs)
-        self.inputs = ('x', 'y')
-        self.outputs = ('label',)
+        super().__init__(mapper, _no_label, name=name, **kwargs)
+        self.inputs = ("x", "y")
+        self.outputs = ("label",)
 
     def evaluate(self, *args):
         args = tuple([_toindex(a) for a in args])
         try:
             result = self._mapper[args[::-1]]
         except IndexError as e:
-            raise LabelMapperArrayIndexingError(e)
+            raise LabelMapperArrayIndexingError(e) from e
         return result
 
     @classmethod
@@ -233,7 +306,7 @@ class LabelMapperArray(_LabelMapper):
         ...           }
         >>> mapper = LabelMapperArray.from_vertices((2400, 2400), regions)
 
-        """
+        """  # noqa: E501
         labels = np.array(list(regions.keys()))
         mask = np.zeros(shape, dtype=labels.dtype)
 
@@ -245,7 +318,6 @@ class LabelMapperArray(_LabelMapper):
 
 
 class LabelMapperDict(_LabelMapper):
-
     """
     Maps a number to a transform, which when evaluated returns a label.
 
@@ -269,6 +341,7 @@ class LabelMapperDict(_LabelMapper):
     name : str
         The name of this transform.
     """
+
     standard_broadcasting = False
 
     linear = False
@@ -276,18 +349,20 @@ class LabelMapperDict(_LabelMapper):
 
     n_outputs = 1
 
-    def __init__(self, inputs, mapper, inputs_mapping=None, atol=10**-8, name=None, **kwargs):
+    def __init__(
+        self, inputs, mapper, inputs_mapping=None, atol=10**-8, name=None, **kwargs
+    ):
         self._atol = atol
         _no_label = 0
         self._inputs = inputs
         self._n_inputs = len(inputs)
-        if not all([m.n_outputs == 1 for m in mapper.values()]):
-            raise TypeError("All transforms in mapper must have one output.")
-        self._input_units_strict = {key: False for key in self._inputs}
-        self._input_units_allow_dimensionless = {key: False for key in self._inputs}
-        super(LabelMapperDict, self).__init__(mapper, _no_label, inputs_mapping,
-                                              name=name, **kwargs)
-        self.outputs = ('labels',)
+        if not all(m.n_outputs == 1 for m in mapper.values()):
+            msg = "All transforms in mapper must have one output."
+            raise TypeError(msg)
+        self._input_units_strict = dict.fromkeys(self._inputs, False)
+        self._input_units_allow_dimensionless = dict.fromkeys(self._inputs, False)
+        super().__init__(mapper, _no_label, inputs_mapping, name=name, **kwargs)
+        self.outputs = ("labels",)
 
     @property
     def n_inputs(self):
@@ -342,7 +417,6 @@ class LabelMapperDict(_LabelMapper):
 
 
 class LabelMapperRange(_LabelMapper):
-
     """
     The structure this class uses maps a range of values to a transform.
     Given an input value it finds the range the value falls in and returns
@@ -367,6 +441,7 @@ class LabelMapperRange(_LabelMapper):
     name : str
         The name of this transform.
     """
+
     standard_broadcasting = False
 
     n_outputs = 1
@@ -376,17 +451,18 @@ class LabelMapperRange(_LabelMapper):
 
     def __init__(self, inputs, mapper, inputs_mapping=None, name=None, **kwargs):
         if self._has_overlapping(np.array(list(mapper.keys()))):
-            raise ValueError("Overlapping ranges of values are not supported.")
+            msg = "Overlapping ranges of values are not supported."
+            raise ValueError(msg)
         self._inputs = inputs
         self._n_inputs = len(inputs)
         _no_label = 0
-        if not all([m.n_outputs == 1 for m in mapper.values()]):
-            raise TypeError("All transforms in mapper must have one output.")
-        self._input_units_strict = {key: False for key in self._inputs}
-        self._input_units_allow_dimensionless = {key: False for key in self._inputs}
-        super(LabelMapperRange, self).__init__(mapper, _no_label, inputs_mapping,
-                                               name=name, **kwargs)
-        self.outputs = ('labels',)
+        if not all(m.n_outputs == 1 for m in mapper.values()):
+            msg = "All transforms in mapper must have one output."
+            raise TypeError(msg)
+        self._input_units_strict = dict.fromkeys(self._inputs, False)
+        self._input_units_allow_dimensionless = dict.fromkeys(self._inputs, False)
+        super().__init__(mapper, _no_label, inputs_mapping, name=name, **kwargs)
+        self.outputs = ("labels",)
 
     @property
     def n_inputs(self):
@@ -415,16 +491,11 @@ class LabelMapperRange(_LabelMapper):
         start = ranges[:, 0]
         end = ranges[:, 1]
         start.sort()
-        l = []
-        for v in start:
-            l.append([v, d[v]])
-        l = np.array(l)
-        start = np.roll(l[:, 0], -1)
-        end = l[:, 1]
-        if any((end - start)[:-1] > 0) or any(start[-1] > end):
-            return True
-        else:
-            return False
+        values = [[v, d[v]] for v in start]
+        values = np.array(values)
+        start = np.roll(values[:, 0], -1)
+        end = values[:, 1]
+        return bool(any((end - start)[:-1] > 0) or any(start[-1] > end))
 
     # move this to utils?
     def _find_range(self, value_range, value):
@@ -447,11 +518,11 @@ class LabelMapperRange(_LabelMapper):
         a, b = value_range[:, 0], value_range[:, 1]
         ind = np.logical_and(value >= a, value <= b).nonzero()[0]
         if ind.size > 1:
-            raise ValueError("There are overlapping ranges.")
-        elif ind.size == 0:
+            msg = "There are overlapping ranges."
+            raise ValueError(msg)
+        if ind.size == 0:
             return None
-        else:
-            return ind.item()
+        return ind.item()
 
     def evaluate(self, *args):
         shape = args[0].shape
@@ -471,9 +542,9 @@ class LabelMapperRange(_LabelMapper):
         for val_range in value_ranges:
             temp = keys.copy()
             temp[nan_ind] = np.nan
-            temp = np.where(np.logical_or(temp <= val_range[0],
-                                          temp >= val_range[1]),
-                            np.nan, temp)
+            temp = np.where(
+                np.logical_or(temp <= val_range[0], temp >= val_range[1]), np.nan, temp
+            )
             ind = ~np.isnan(temp)
 
             if ind.any():
@@ -483,12 +554,13 @@ class LabelMapperRange(_LabelMapper):
                 continue
         res.shape = shape
         if len(np.nonzero(res)[0]) == 0:
-            warnings.warn("All data is outside the valid range - {0}.".format(self.name))
+            warnings.warn(
+                f"All data is outside the valid range - {self.name}.", stacklevel=2
+            )
         return res
 
 
 class RegionsSelector(Model):
-
     """
     This model defines discontinuous transforms.
     It maps inputs to their corresponding transforms.
@@ -512,13 +584,22 @@ class RegionsSelector(Model):
     name : str
         The name of this transform.
     """
+
     standard_broadcasting = False
 
     linear = False
     fittable = False
 
-    def __init__(self, inputs, outputs, selector, label_mapper, undefined_transform_value=np.nan,
-                 name=None, **kwargs):
+    def __init__(
+        self,
+        inputs,
+        outputs,
+        selector,
+        label_mapper,
+        undefined_transform_value=np.nan,
+        name=None,
+        **kwargs,
+    ):
         self._inputs = inputs
         self._outputs = outputs
         self._n_inputs = len(inputs)
@@ -527,11 +608,28 @@ class RegionsSelector(Model):
         self._undefined_transform_value = undefined_transform_value
         self._selector = selector  # copy.deepcopy(selector)
 
-        if " " in selector.keys() or 0 in selector.keys():
-            raise ValueError('"0" and " " are not allowed as keys.')
-        self._input_units_strict = {key: False for key in self._inputs}
-        self._input_units_allow_dimensionless = {key: False for key in self._inputs}
-        super(RegionsSelector, self).__init__(n_models=1, name=name, **kwargs)
+        if " " in selector or 0 in selector:
+            msg = '"0" and " " are not allowed as keys.'
+            raise ValueError(msg)
+        self._input_units_strict = dict.fromkeys(self._inputs, False)
+        self._input_units_allow_dimensionless = dict.fromkeys(self._inputs, False)
+        super().__init__(n_models=1, name=name, **kwargs)
+        # Validate uses_quantity at init time for nicer error message
+        _ = self.uses_quantity
+
+    @property
+    def uses_quantity(self):
+        all_uses_quantity = [t.uses_quantity for t in self._selector.values()]
+        not_all_uses_quantity = [not uq for uq in all_uses_quantity]
+        if all(all_uses_quantity):
+            return True
+        if not_all_uses_quantity:
+            return False
+        msg = (
+            "You can not mix models which use quantity and do not use "
+            "quantity inside a RegionSelector"
+        )
+        raise ValueError(msg)
 
     def set_input(self, rid):
         """
@@ -539,8 +637,8 @@ class RegionsSelector(Model):
         """
         if rid in self._selector:
             return self._selector[rid]
-        else:
-            raise RegionError("Region {0} not found".format(rid))
+        msg = f"Region {rid} not found"
+        raise RegionError(msg)
 
     def inverse(self):
         if self.label_mapper.inverse is not None:
@@ -548,14 +646,20 @@ class RegionsSelector(Model):
                 transforms_inv = {}
                 for rid in self._selector:
                     transforms_inv[rid] = self._selector[rid].inverse
-            except AttributeError:
-                raise NotImplementedError("The inverse of all regions must be defined"
-                                          "for RegionsSelector to have an inverse.")
-            return self.__class__(self.outputs, self.inputs, transforms_inv,
-                                  self.label_mapper.inverse)
-        else:
-            raise NotImplementedError("The label mapper must have an inverse "
-                                      "for RegionsSelector to have an inverse.")
+            except AttributeError as err:
+                msg = (
+                    "The inverse of all regions must be defined"
+                    "for RegionsSelector to have an inverse."
+                )
+                raise NotImplementedError(msg) from err
+            return self.__class__(
+                self.outputs, self.inputs, transforms_inv, self.label_mapper.inverse
+            )
+        msg = (
+            "The label mapper must have an inverse "
+            "for RegionsSelector to have an inverse."
+        )
+        raise NotImplementedError(msg)
 
     def evaluate(self, *args):
         """
@@ -569,7 +673,9 @@ class RegionsSelector(Model):
         rids = self.label_mapper(*args).flatten()
         # Raise an error if all pixels are outside regions
         if (rids == self.label_mapper.no_label).all():
-            warnings.warn("The input positions are not inside any region.")
+            warnings.warn(
+                "The input positions are not inside any region.", stacklevel=2
+            )
 
         # Create output arrays and set any pixels not within regions to
         # "undefined_transform_value"
@@ -583,14 +689,16 @@ class RegionsSelector(Model):
         uniq = get_unique_regions(rids)
 
         for rid in uniq:
-            ind = (rids == rid)
+            ind = rids == rid
             inputs = [a[ind] for a in args]
             if rid in self._selector:
                 result = self._selector[rid](*inputs)
             else:
                 # If there's no transform for a label, return np.nan
-                result = [np.empty(inputs[0].shape) +
-                          self._undefined_transform_value for i in range(self.n_outputs)]
+                result = [
+                    np.full(inputs[0].shape, self._undefined_transform_value)
+                    for i in range(self.n_outputs)
+                ]
             for j in range(self.n_outputs):
                 outputs[j][ind] = result[j]
         return outputs
@@ -666,22 +774,27 @@ class LabelMapper(_LabelMapper):
 
     n_outputs = 1
 
-    def __init__(self, inputs, mapper, no_label=np.nan, inputs_mapping=None, name=None, **kwargs):
+    def __init__(
+        self, inputs, mapper, no_label=np.nan, inputs_mapping=None, name=None, **kwargs
+    ):
         self._no_label = no_label
         self._inputs = inputs
         self._n_inputs = len(inputs)
-        self._outputs = tuple(['x{0}'.format(ind) for ind in list(range(mapper.n_outputs))])
+        self._outputs = tuple([f"x{ind}" for ind in list(range(mapper.n_outputs))])
         if isinstance(inputs_mapping, tuple):
             inputs_mapping = astmodels.Mapping(inputs_mapping)
-        elif inputs_mapping is not None and not isinstance(inputs_mapping, astmodels.Mapping):
-            raise TypeError("inputs_mapping must be an instance of astropy.modeling.Mapping.")
+        elif inputs_mapping is not None and not isinstance(
+            inputs_mapping, astmodels.Mapping
+        ):
+            msg = "inputs_mapping must be an instance of astropy.modeling.Mapping."
+            raise TypeError(msg)
 
         self._inputs_mapping = inputs_mapping
         self._mapper = mapper
-        self._input_units_strict = {key: False for key in self._inputs}
-        self._input_units_allow_dimensionless = {key: False for key in self._inputs}
+        self._input_units_strict = dict.fromkeys(self._inputs, False)
+        self._input_units_allow_dimensionless = dict.fromkeys(self._inputs, False)
         super(_LabelMapper, self).__init__(name=name, **kwargs)
-        self.outputs = ('label',)
+        self.outputs = ("label",)
 
     @property
     def inputs(self):
